@@ -81,8 +81,7 @@ from pymc.distributions.distribution import (
     DensityDist,
     Discrete,
     Distribution,
-    NoDistribution,
-    SymbolicDistribution,
+    SymbolicRandomVariable,
 )
 from pymc.distributions.mixture import Mixture, NormalMixture
 from pymc.distributions.multivariate import (
@@ -105,10 +104,13 @@ from pymc.distributions.simulator import Simulator
 from pymc.distributions.timeseries import (
     AR,
     GARCH11,
+    EulerMaruyama,
     GaussianRandomWalk,
     MvGaussianRandomWalk,
     MvStudentTRandomWalk,
+    RandomWalk,
 )
+from pymc.distributions.truncated import Truncated
 
 __all__ = [
     "Uniform",
@@ -154,10 +156,9 @@ __all__ = [
     "OrderedProbit",
     "DensityDist",
     "Distribution",
-    "SymbolicDistribution",
+    "SymbolicRandomVariable",
     "Continuous",
     "Discrete",
-    "NoDistribution",
     "MvNormal",
     "MatrixNormal",
     "KroneckerNormal",
@@ -171,11 +172,13 @@ __all__ = [
     "WishartBartlett",
     "LKJCholeskyCov",
     "LKJCorr",
-    "AR",
     "AsymmetricLaplace",
+    "RandomWalk",
     "GaussianRandomWalk",
     "MvGaussianRandomWalk",
     "MvStudentTRandomWalk",
+    "AR",
+    "EulerMaruyama",
     "GARCH11",
     "SkewNormal",
     "Mixture",
@@ -190,6 +193,7 @@ __all__ = [
     "Rice",
     "Moyal",
     "Simulator",
+    "Truncated",
     "Censored",
     "CAR",
     "PolyaGamma",
