@@ -11,11 +11,11 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
+import pickle
+import threading
 import unittest
 import warnings
 
-from functools import reduce
-
 import aesara
 import aesara.sparse as sparse
 import aesara.tensor as at
@@ -38,9 +38,10 @@ import pymc as pm
 from pymc import Deterministic, Potential
 from pymc.blocking import DictToArrayBijection, RaveledVars
 from pymc.distributions import Normal, transforms
-from pymc.exceptions import ShapeError, ShapeWarning
-from pymc.model import Point, ValueGradFunction
+from pymc.exceptions import ImputationWarning, ShapeError, ShapeWarning
+from pymc.model import Point, ValueGradFunction, modelcontext
 from pymc.tests.helpers import SeededTest
+from pymc.tests.models import simple_model
 
 
 class NewModel(pm.Model):
@@ -341,7 +342,8 @@ class TestValueGradFunction(unittest.TestCase):
         X = np.ma.masked_values(X, value=-1)
         with pm.Model() as m:
             x1 = pm.Uniform("x1", 0.0, 1.0)
-            x2 = pm.Bernoulli("x2", x1, observed=X)
+            with pytest.warns(ImputationWarning):
+                x2 = pm.Bernoulli("x2", x1, observed=X)
 
         gf = m.logp_dlogp_function()
         gf._extra_are_set = True
@@ -353,7 +355,8 @@ class TestValueGradFunction(unittest.TestCase):
 
         res = [gf(DictToArrayBijection.map(Point(pnt, model=m))) for i in range(5)]
 
-        assert reduce(lambda x, y: np.array_equal(x, y) and y, res) is not False
+        # Assert that all the elements of res are equal
+        assert res[1:] == res[:-1]
 
     def test_aesara_switch_broadcast_edge_cases_1(self):
         # Tests against two subtle issues related to a previous bug in Theano
@@ -443,24 +446,39 @@ def test_tempered_logp_dlogp():
     npt.assert_allclose(func_temp_nograd(x), func_temp(x)[0])
 
 
-def test_model_pickle(tmpdir):
-    """Tests that PyMC models are pickleable"""
-    with pm.Model() as model:
-        x = pm.Normal("x")
-        pm.Normal("y", observed=1)
+class TestPickling:
+    def test_model_pickle(self, tmpdir):
+        """Tests that PyMC models are pickleable"""
+        with pm.Model() as model:
+            x = pm.Normal("x")
+            pm.Normal("y", observed=1)
 
-    cloudpickle.loads(cloudpickle.dumps(model))
+        cloudpickle.loads(cloudpickle.dumps(model))
 
+    def test_model_pickle_deterministic(self, tmpdir):
+        """Tests that PyMC models are pickleable"""
+        with pm.Model() as model:
+            x = pm.Normal("x")
+            z = pm.Normal("z")
+            pm.Deterministic("w", x / z)
+            pm.Normal("y", observed=1)
 
-def test_model_pickle_deterministic(tmpdir):
-    """Tests that PyMC models are pickleable"""
-    with pm.Model() as model:
-        x = pm.Normal("x")
-        z = pm.Normal("z")
-        pm.Deterministic("w", x / z)
-        pm.Normal("y", observed=1)
+        cloudpickle.loads(cloudpickle.dumps(model))
 
-    cloudpickle.loads(cloudpickle.dumps(model))
+    def setup_method(self):
+        _, self.model, _ = simple_model()
+
+    def test_model_roundtrip(self):
+        m = self.model
+        for proto in range(pickle.HIGHEST_PROTOCOL + 1):
+            try:
+                s = cloudpickle.dumps(m, proto)
+                cloudpickle.loads(s)
+            except Exception:
+                raise AssertionError(
+                    "Exception while trying roundtrip with pickle protocol %d:\n" % proto
+                    + "".join(traceback.format_exc())
+                )
 
 
 def test_model_vars():
@@ -522,7 +540,8 @@ def test_make_obs_var():
     del fake_model.named_vars[fake_distribution.name]
 
     # Here the RandomVariable is split into observed/imputed and a Deterministic is returned
-    masked_output = fake_model.make_obs_var(fake_distribution, masked_array_input, None, None)
+    with pytest.warns(ImputationWarning):
+        masked_output = fake_model.make_obs_var(fake_distribution, masked_array_input, None, None)
     assert masked_output != fake_distribution
     assert not isinstance(masked_output, RandomVariable)
     # Ensure it has missing values
@@ -723,7 +742,7 @@ def test_shapeerror_from_resize_immutable_dim_from_RV():
     Even if the variable being updated is a SharedVariable and has other
     dimensions that are mutable.
     """
-    with pm.Model() as pmodel:
+    with pm.Model(coords={"fixed": range(3)}) as pmodel:
         pm.Normal("a", mu=[1, 2, 3], dims="fixed")
         assert isinstance(pmodel.dim_lengths["fixed"], TensorVariable)
 
@@ -732,7 +751,8 @@ def test_shapeerror_from_resize_immutable_dim_from_RV():
         # This is fine because the "fixed" dim is not resized
         pmodel.set_data("m", [[1, 2, 3], [3, 4, 5]])
 
-    with pytest.raises(ShapeError, match="was initialized from 'a'"):
+    msg = "The 'm' variable already had 3 coord values defined for its fixed dimension"
+    with pytest.raises(ValueError, match=msg):
         # Can't work because the "fixed" dimension is linked to a
         # TensorVariable with constant shape.
         # Note that the new data tries to change both dimensions
@@ -807,7 +827,7 @@ def test_set_dim():
 
 
 def test_set_dim_with_coords():
-    """Test the concious re-sizing of dims created through add_coord() with coord value."""
+    """Test the conscious re-sizing of dims created through add_coord() with coord value."""
     with pm.Model() as pmodel:
         pmodel.add_coord("mdim", mutable=True, length=2, values=["A", "B"])
         a = pm.Normal("a", dims="mdim")
@@ -885,6 +905,17 @@ def test_set_data_indirect_resize_with_coords():
         pmodel.set_data("mdata", [1, 2], coords=dict(mdim=[1, 2, 3]))
 
 
+def test_set_data_constant_shape_error():
+    with pm.Model() as pmodel:
+        x = pm.Normal("x", size=7)
+        pmodel.add_coord("weekday", length=x.shape[0])
+        pm.MutableData("y", np.arange(7), dims="weekday")
+
+    msg = "because the dimension was initialized from 'x' which is not a shared variable"
+    with pytest.raises(ShapeError, match=msg):
+        pmodel.set_data("y", np.arange(10))
+
+
 def test_model_logpt_deprecation_warning():
     with pm.Model() as m:
         x = pm.Normal("x", 0, 1, size=2)
@@ -1036,3 +1067,82 @@ def test_model_parent_set_programmatically():
         y = pm.Normal("y")
 
     assert "y" in model.named_vars
+
+
+class TestModelContext:
+    def test_thread_safety(self):
+        """Regression test for issue #1552: Thread safety of model context manager
+
+        This test creates two threads that attempt to construct two
+        unrelated models at the same time.
+        For repeatable testing, the two threads are syncronised such
+        that thread A enters the context manager first, then B,
+        then A attempts to declare a variable while B is still in the context manager.
+        """
+        aInCtxt, bInCtxt, aDone = (threading.Event() for _ in range(3))
+        modelA = pm.Model()
+        modelB = pm.Model()
+
+        def make_model_a():
+            with modelA:
+                aInCtxt.set()
+                bInCtxt.wait()
+                Normal("a", 0, 1)
+            aDone.set()
+
+        def make_model_b():
+            aInCtxt.wait()
+            with modelB:
+                bInCtxt.set()
+                aDone.wait()
+                Normal("b", 0, 1)
+
+        threadA = threading.Thread(target=make_model_a)
+        threadB = threading.Thread(target=make_model_b)
+        threadA.start()
+        threadB.start()
+        threadA.join()
+        threadB.join()
+        # now let's see which model got which variable
+        # previous to #1555, the variables would be swapped:
+        # - B enters it's model context after A, but before a is declared -> a goes into B
+        # - A leaves it's model context before B attempts to declare b. A's context manager
+        #   takes B from the stack, such that b ends up in model A
+        assert (
+            list(modelA.named_vars),
+            list(modelB.named_vars),
+        ) == (["a"], ["b"])
+
+
+def test_mixed_contexts():
+    modelA = pm.Model()
+    modelB = pm.Model()
+    with pytest.raises((ValueError, TypeError)):
+        modelcontext(None)
+    with modelA:
+        with modelB:
+            assert pm.Model.get_context() == modelB
+            assert modelcontext(None) == modelB
+        assert pm.Model.get_context() == modelA
+        assert modelcontext(None) == modelA
+    assert pm.Model.get_context(error_if_none=False) is None
+    with pytest.raises(TypeError):
+        pm.Model.get_context(error_if_none=True)
+    with pytest.raises((ValueError, TypeError)):
+        modelcontext(None)
+
+
+class TestProfile:
+    def setup_method(self):
+        _, self.model, _ = simple_model()
+
+    def test_profile_model(self):
+        assert self.model.profile(self.model.logp()).fct_call_time > 0
+
+    def test_profile_variable(self):
+        rv = self.model.basic_RVs[0]
+        assert self.model.profile(self.model.logp(vars=[rv], sum=False)).fct_call_time
+
+    def test_profile_count(self):
+        count = 1005
+        assert self.model.profile(self.model.logp(), n=count).fct_callcount == count
